﻿-- Author      : Malex
-- Create Date : 5/24/2009

-- Addon setup
--
WowLife = LibStub("AceAddon-3.0"):NewAddon("WowLife-1.0.0", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0");

-- Localization import
--
local L = LibStub("AceLocale-3.0"):GetLocale("WowLife", true);

-- Globals
--
WowLife.Version = "1.0.0";
WowLife.IsDebug = true;
WowLife.TotalPlayedTime = nil;
WowLife.PlayerAge = nil;

-- Default options
--
WowLife.DefaultOptions = {
	global = {
		Version = WowLife.Version,
		BirthDay = {
			month = date("*t").month,
			day = date("*t").day,
			year = date("*t").year
		},
		Precision = 8,
		ShowWindow = true,
		Characters = {}
	}
};

-- Config UI
--
WowLife.Config = {
    name = "WowLife",
    type = "group",  
	args = {
		desc = {
			type = "description",
			name = L["OptionsDesc"],
			order = 1
		},
		about = {
			name = "About",
			type = "group", 
			inline = true,
			order = 2,
			args = {
				version = {
					order = 1,
					type = "description",			
					name = "|cffffd700".."Version"..": ".._G["GREEN_FONT_COLOR_CODE"]..WowLife.Version,
					cmdHidden = true
				},
				author = {
					order = 2,
					type = "description",
					name = "|cffffd700".."Author"..": ".."|cffff8c00".."Malex (Blade's Edge)",
					cmdHidden = true
				}
			}
		},
		usageHeader = {
			type = "header",
			name = L["UsageHeader"],
			order = 3,
		},				
		usage = {
			type = "description",
			name = L["Usage"],
			order = 4
		},
		configHeader = {
			type = "header",
			name = L["ConfigHeader"],
			order = 5,
		},	
		showWindow = {
			type = "toggle", 
			name = L["ShowWindow"],
			desc = L["ShowWindowTooltip"],
			width = "full",
			order = 6,
			get = function() return WowLife.db.global.ShowWindow end,
			set = function(_, value)
				WowLife.db.global.ShowWindow = value;
				if(value) then
					WowLifeFrame:Show();
				else
					WowLifeFrame:Hide();
				end
			end,
		},								
		birth = {
			type = "input",
			name = L["BirthDay"],
			desc = L["BirthDayTooltip"],
			width = "full",
			order = 7,
			get = function() 
				return date("%m/%d/%Y", time(WowLife.db.global.BirthDay));
			end,
			set = function(_, value) 
				local month, day, year = string.match(value, "^(%d%d)/(%d%d)/(%d%d%d%d)$");
				WowLife.db.global.BirthDay.month = tonumber(month);
				WowLife.db.global.BirthDay.day = tonumber(day);
				WowLife.db.global.BirthDay.year = tonumber(year);
			end,
			validate = function(_, value) 
				local month, day, year = string.match(value, "^(%d%d)/(%d%d)/(%d%d%d%d)$");				
				if(month == nil or day == nil or year == nil) then
					return false;
				end
				
				month = tonumber(month);
				day = tonumber(day);
				year = tonumber(year);
				
				if(month < 1 or month > 12) then
					return false;
				end
				
				if(day < 1 or day > 31) then
					return false;
				end
				
				if(difftime(time(date("*t")), time({year = year, month = month, day = day})) < 0) then
					return false;
				end 						
				
				return true;
			end
		},
		precision = {
			type = "range", 
			name = L["Precision"],
			desc = L["PrecisionTooltip"],
			width = "full",
			order = 8,
			min = 1, 
			max = 8, 
			step = 1,
			get = function() return WowLife.db.global.Precision end,
			set = function(_, value) WowLife.db.global.Precision = value; end
		}				
	}
};



--------------------------------------------------------------------------------
--
-- Method: OnInitialize
-- Description: Called during the initialization of the addon
--
--------------------------------------------------------------------------------
function WowLife:OnInitialize()
		
	self:Print("WowLife initializing");
	
	-- Initialize external libraries
	--
	self.db = LibStub("AceDB-3.0"):New("WowLifeDB", self.DefaultOptions);
			
	-- Register the slash handler
	--
	self:RegisterChatCommand("wowlife", "SlashCommandHandler");
	
	-- Event Registration
	--	
	self:RegisterEvent("TIME_PLAYED_MSG", "OnEvent");
    self:RegisterEvent("VARIABLES_LOADED", "OnEvent");
      	
	-- Register the WowLife config screens with blizz config ui
	--	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("WowLife", self.Config);
	self.ConfigUI = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("WowLife", "WowLife");	
	
	WowLifeFrame:SetScale(0.7);
	
	-- LibDataBroker
	--
	WowLife.LDB = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(
		"WowLife", 
		{
			type = "data source",
			icon = "Interface\\ICONS\\Achievement_BG_AB_kill_in_mine",
			text = "",
			OnClick = function(frame, button) InterfaceOptionsFrame_OpenToCategory(WowLife.ConfigUI); end
		}
	);	
end


--------------------------------------------------------------------------------
--
-- Method: SlashCommandHandler
-- Description: Handles the slash commands for the addon
-- Arguments:
--	msg - [in] The text arguments to the command
--
--------------------------------------------------------------------------------
function WowLife:SlashCommandHandler(msg)	
	
	if(strlower(msg) == L["config"]) then
		InterfaceOptionsFrame_OpenToCategory(WowLife.ConfigUI);
	elseif(strlower(msg) == L["toggle"]) then
		if(WowLifeFrame:IsShown()) then
			self.db.global.ShowWindow = false;
			WowLifeFrame:Hide();
		else
			self.db.global.ShowWindow = true;
			WowLifeFrame:Show();
		end
	else
		-- Print the total % of life in wow
		--
		self:Print(L["TotalTimeMessage"] .. self:GetWowLifePercent());
	end
end

--------------------------------------------------------------------------------
--
-- Method: GetWowLifePercent
-- Description: Returns the percent of the user's life they have spent playing
--	wow in a string
--
--------------------------------------------------------------------------------
function WowLife:GetWowLifePercent()

	local secondsSinceBirth = difftime(time(date("*t")), time(self.db.global.BirthDay));	
	local percent = (self.TotalPlayedTime / secondsSinceBirth) * 100;
	
	if(percent > 100) then
		return L["NeedsConfigure"];
	else
		return string.format("%.".. self.db.global.Precision .."f%%", percent);
	end
	
end

--------------------------------------------------------------------------------
--
-- Method: OnEvent
-- Description: The event handler for the addon
--
--------------------------------------------------------------------------------
function WowLife:OnEvent(eventName, ...)

	if(eventName == "TIME_PLAYED_MSG") then
			
		if(self.TotalPlayedTime == nil) then
		
			-- Replace the saved total time for the current character
			--
			local name = UnitName("player");
			local secondsPlayed = arg1;
			self.db.global.Characters[name] = arg1;
			
			-- Calculate the total played time over all saved chars
			--
			self.TotalPlayedTime = 0;
			for index, value in pairs(self.db.global.Characters) do
				self.TotalPlayedTime = self.TotalPlayedTime + value;
			end
			
			-- Schedule a timer to update the player's age and time played each second
			--
			self.UpdateTimer = self:ScheduleRepeatingTimer("OnUpdateTimer", 1);
									
		end
	
	elseif(eventName == "VARIABLES_LOADED") then
	
		-- Load saved variables
		--
		self:LoadOptions();
		
		-- Get the latest played time for the current character
		--
		RequestTimePlayed();
	
	end	

end

--------------------------------------------------------------------------------
--
-- Method: OnUpdateTimer
-- Description: 
--
--------------------------------------------------------------------------------
function WowLife:OnUpdateTimer(context)
	
	-- Increment the total played time by one second
	--
	self.TotalPlayedTime = self.TotalPlayedTime + 1;
	
	local wowLifePercent = self:GetWowLifePercent();
	
	WowLifeFrame_WowLife:SetText(L["WowLifeMessage"] .. wowLifePercent);
	WowLife.LDB.text = wowLifePercent;
	
end


--------------------------------------------------------------------------------
--
-- Method: LoadOptions
-- Description: Loads the default options if they do not exist
--
--------------------------------------------------------------------------------
function WowLife:LoadOptions()

	-- Load the defaults if there are no saved variables or if there is a new version
	--
	if(self.db.global.Version ~= self.Version) then		
		self.db = self.DefaultOptions;
	end
	
	if(self.db.global.ShowWindow == true) then
		WowLifeFrame:Show();
	end	
			
end

